
open Graph

(* directed graphs with integer coordinates and integer labels on edges *)

module IntInt = struct 
  type t = int * int 
end
module Int = struct 
  type t = int 
  let compare = compare 
  let hash = Hashtbl.hash 
  let equal = (=)
  let default = 0
end
module G = Imperative.Digraph.AbstractLabeled(IntInt)(Int)

let n_ = ref 10
let prob_ = ref 0.5
let () = 
  Arg.parse
      ["-v", Arg.Int (fun i -> n_ := i), 
       " <int>  number of vertices";
       "-prob", Arg.Float (fun f -> prob_ := f), 
       " <float>  probability to discrad an edge";
      ]
      (fun _ -> ())
      "usage: demo_planar <options>"
let n = !n_
let prob = !prob_

(* a random digraph with n vertices *)
let () = Random.self_init ()
module R = Rand.Planar.I(G)
let g = R.graph ~xrange:(0,800) ~yrange:(0,600) ~prob n
let () = Printf.printf "#V = %d #E = %d\n" (G.nb_vertex g) (G.nb_edges g)

open Graphics
let () = open_graph " 800x600"

let round f = truncate (f +. 0.5)
let pi = 4.0 *. atan 1.0

let draw_arrow (xu,yu) (xv,yv) =
  let dx = float (xv - xu) in
  let dy = float (yv - yu) in
  let alpha = atan2 dy dx in
  let r = sqrt (dx *. dx +. dy *. dy) in
  let ra = 10. in
  let xs, ys = float xu +. 5. *. dx /. r, float yu +. 5. *. dy /. r in
  let xd, yd = float xv -. 5. *. dx /. r, float yv -. 5. *. dy /. r in
  let coords theta =
    round (xd +. ra *. cos (pi +. alpha +. theta)),
    round (yd +. ra *. sin (pi +. alpha +. theta))
  in
  moveto (round xs) (round ys);
  lineto (round xd) (round yd);
  let x1,y1 = coords (pi /. 6.) in
  moveto (round xd) (round yd); lineto x1 y1;
  let x2,y2 = coords (-. pi /. 6.) in
  moveto (round xd) (round yd); lineto x2 y2

let draw_graph g = 
  clear_graph ();
  set_color red;
  G.iter_vertex
    (fun v ->
       let (x,y) = G.V.label v in
       draw_circle x y 3)
    g;
  set_color black;
  G.iter_edges
    (fun v1 v2 -> draw_arrow (G.V.label v1) (G.V.label v2))
    g

let () = draw_graph g
let _ = ignore (Graphics.wait_next_event [ Graphics.Key_pressed ])
let () = close_graph ()

